#!/usr/bin/perl
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted
# material is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Milan Zazrivec

use warnings;
use strict;

print "Running IPv4 network gateway reachibility test\n\n".
      "Test will scan IPv4 routing table for configured gateways and test\n".
      "their reachibility using ICMP ping and/or ARP ping.\n\n";

my $line;
my ($flags, $gw);
my $p = qr/^(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)$/x;
my @gateways_tmp;
my @gateways = ();
my %gw_uniq_list;
my $address;
my $errors = 0;

# we'll parse /proc/net/route for all records with active RTF_GATEWAY flag
die "$!\n" unless open FILE, "/proc/net/route";
while ($line = <FILE>)
{
    chomp $line;
    next unless (($line =~ $p) and ($. != 1));
    $gw = $3;
    $flags = $4;

    # /usr/include/net/route.h:
    # #define RTF_GATEWAY     0x0002          /* Destination is a gateway. */
    next unless (($flags & 2) == 2);
    ## fast, but ugly-looking C-like solution
    # my $qwd = hex($gw);
    # my $quad1 = $qwd%256;
    # my $quad2 = ($qwd>>8)%256;
    # my $quad3 = ($qwd>>16)%256;
    # my $quad4 = ($qwd>>24)%256;
    # my $gw_addr = $quad1.'.'.$quad2.'.'.$quad3.'.'.$quad4;
	
    # little slower, but nice-looking Perl-like solution
    $gw =~ s/([0-9a-fA-F]{8})/join ".", reverse unpack "C*", pack "H*", $1/eig;
    push @gateways_tmp, "$gw";
}
close FILE;

foreach $address (@gateways_tmp)
{
    if (not defined $gw_uniq_list{$address})
    {
        $gw_uniq_list{$address} = 1;
        push @gateways, $address;
    }
}

if (@gateways)
{
    print "Found IPv4 gateways:\n";
    foreach $address (@gateways)
    {
        print "$address\n";
    }
}
else
{
    print "Test did not find any gateways in system's IPv4 routing table.\n".
          "Exiting test.\n";
    exit 0;
}

foreach $address (@gateways)
{
    # the concept is to ping every found gateway using ICMP ping
    # if ICMP doesn't succeed, we'll try arping instead

    print "\nTrying ICMP ping to $address... ";
    if (system("/bin/ping -c 3 $address >& /dev/null") != 0)
    {
        print "Failed.\nTrying ARP ping to $address... ";
        if (system("/sbin/arping -q -c 3 $address") != 0)
        {
            print "Failed.\nGateway $address (IPv4) could not be reached\n";
            ++$errors;
        }
	else
	{
            print "Succeeded.\n";
	}
    }
    else
    {
        print "Succeeded\n";
    }
}

print "\nTest complete: result=PASS\n" if not $errors;
print "\nTest complete: result=FAIL\n" if $errors;

exit $errors;
